@@ -15,7 +15,7 @@ import re
 import sys
 from warnings import warn
 from abc import ABCMeta, abstractmethod
-from typing import Any, Dict, List, Optional, Union, Tuple, Type, cast
+from typing import Any, AnyStr, Dict, List, Optional, Union, Tuple, Type, cast
 
 if sys.version_info < (3, 7):
     re.Pattern = Any
@@ -55,7 +55,7 @@ class _Tag(metaclass=ABCMeta):
         return self.update_impl(spec_obj, context, match_obj, line)
 
     @abstractmethod
-    def update_impl(self, spec_obj, context, match_obj, line):
+    def update_impl(self, spec_obj: "Spec", context: Dict[str, Any], match_obj: re.Match, line: str) -> Tuple["Spec", dict]:
         pass
 
     @staticmethod
@@ -98,7 +98,8 @@ class _SetterMacroDef(_Tag):
     def __init__(self, name: str, pattern_obj: re.Pattern) -> None:
         super().__init__(name, pattern_obj, str)
 
-    def get_namespace(self, spec_obj, context):
+    @abstractmethod
+    def get_namespace(self, spec_obj: "Spec", context: Dict[str, Any]) -> "Spec":
         raise NotImplementedError()
 
     def update_impl(self, spec_obj: "Spec", context: Dict[str, Any], match_obj: re.Match, line: str) -> Tuple["Spec", dict]:
@@ -124,10 +125,10 @@ class _LocalMacroDef(_SetterMacroDef):
 class _MacroDef(_Tag):
     """Parse global macro definitions."""
 
-    def __init__(self, name, pattern_obj):
+    def __init__(self, name: str, pattern_obj: re.Pattern) -> None:
         super().__init__(name, pattern_obj, str)
 
-    def update_impl(self, spec_obj, context, match_obj, line):
+    def update_impl(self, spec_obj: "Spec", context: Dict[str, Any], match_obj: re.Match, line: str) -> Tuple["Spec", dict]:
         name, value = match_obj.groups()
         spec_obj.macros[name] = str(value)
         if name not in _tag_names:
@@ -217,7 +218,7 @@ class _ListAndDict(_Tag):
 class _SplitValue(_NameValue):
     """Parse a (name->value) tag, and at the same time split the tag to a list."""
 
-    def __init__(self, name: str, pattern_obj: re.Pattern, sep: str = None) -> None:
+    def __init__(self, name: str, pattern_obj: re.Pattern, sep: Optional[None] = None) -> None:
         super().__init__(name, pattern_obj)
         self.name_list = f"{name}_list"
         self.sep = sep
@@ -226,24 +227,24 @@ class _SplitValue(_NameValue):
         super().update_impl(spec_obj, context, match_obj, line)
 
         target_obj = _Tag.current_target(spec_obj, context)
-        value = getattr(target_obj, self.name)
-        value = value.split(self.sep)
-        setattr(target_obj, self.name_list, value)
+        value: str = getattr(target_obj, self.name)
+        values = value.split(self.sep)
+        setattr(target_obj, self.name_list, values)
 
         return spec_obj, context
 
 
-def re_tag_compile(tag):
+def re_tag_compile(tag: AnyStr) -> re.Pattern:
     return re.compile(tag, re.IGNORECASE)
 
 
 class _DummyMacroDef(_Tag):
     """Parse global macro definitions."""
 
-    def __init__(self, name, pattern_obj):
+    def __init__(self, name: str, pattern_obj: re.Pattern) -> None:
         super().__init__(name, pattern_obj, str)
 
-    def update_impl(self, spec_obj, context, _, line):
+    def update_impl(self, spec_obj: "Spec", context: Dict[str, Any], _: re.Match, line: str) -> Tuple["Spec", dict]:
         context["line_processor"] = None
         if warnings_enabled:
             warn("Unknown macro: " + line)
@@ -280,7 +281,7 @@ _tags = [
 
 _tag_names = [tag.name for tag in _tags]
 
-_macro_pattern = re.compile(r"%{(\S+?)\}")
+_macro_pattern = re.compile(r"%{(\S+?)\}|%(\w+?)\b")
 
 
 def _parse(spec_obj: "Spec", context: Dict[str, Any], line: str) -> Any:
@@ -352,7 +353,7 @@ class Requirement:
             return self.name == o.name and self.operator == o.operator and self.version == o.version
         return False
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"Requirement('{self.line}')"
 
 
@@ -450,30 +451,30 @@ class Spec:
         assert self.packages
         return dict(zip([package.name for package in self.packages], self.packages))
 
-    @staticmethod
-    def from_file(filename: str) -> "Spec":
+    @classmethod
+    def from_file(cls, filename: str) -> "Spec":
         """Creates a new Spec object from a given file.
 
         :param filename: The path to the spec file.
         :return: A new Spec object.
         """
 
-        spec = Spec()
+        spec = cls()
         with open(filename, "r", encoding="utf-8") as f:
             parse_context = {"current_subpackage": None}
             for line in f:
                 spec, parse_context = _parse(spec, parse_context, line.rstrip())
         return spec
 
-    @staticmethod
-    def from_string(string: str) -> "Spec":
+    @classmethod
+    def from_string(cls, string: str) -> "Spec":
         """Creates a new Spec object from a given string.
 
         :param string: The contents of a spec file.
         :return: A new Spec object.
         """
 
-        spec = Spec()
+        spec = cls()
         parse_context = {"current_subpackage": None}
         for line in string.splitlines():
             spec, parse_context = _parse(spec, parse_context, line)
@@ -493,6 +494,12 @@ def replace_macros(string: str, spec: Spec) -> str:
     """
     assert isinstance(spec, Spec)
 
+    def _first_not_none(tup: Tuple[Any, ...]) -> Any:
+        for i in tup:
+            if i is not None:
+                return i
+        raise AssertionError("All elements in tuple are None")
+
     def _is_conditional(macro: str) -> bool:
         return macro.startswith("?") or macro.startswith("!")
 
@@ -503,9 +510,11 @@ def replace_macros(string: str, spec: Spec) -> str:
             return False
         raise Exception("Given string is not a conditional macro")
 
-    def _macro_repl(match):
+    def _macro_repl(match: re.Match) -> str:
         # pylint: disable=too-many-return-statements
-        macro_name = match.group(1)
+        groups = match.groups()
+        macro_name: str = _first_not_none(groups)
+        assert macro_name
         if _is_conditional(macro_name) and spec:
             parts = macro_name[1:].split(sep=":", maxsplit=1)
             assert parts
@@ -514,7 +523,7 @@ def replace_macros(string: str, spec: Spec) -> str:
                     if len(parts) == 2:
                         return parts[1]
 
-                    return spec.macros.get(parts[0], getattr(spec, parts[0], None))
+                    return spec.macros.get(parts[0], getattr(spec, parts[0]))
 
                 return ""
 
@@ -522,7 +531,7 @@ def replace_macros(string: str, spec: Spec) -> str:
                 if len(parts) == 2:
                     return parts[1]
 
-                return spec.macros.get(parts[0], getattr(spec, parts[0], None))
+                return spec.macros.get(parts[0], getattr(spec, parts[0]))
 
             return ""
 
